@@ -22,7 +22,8 @@
 #include "container.h"
 #include "vector.h"
 #include <QStringList>
-#include "localize.h"
+#include <QCoreApplication>
+
 #include "list.h"
 #include "variable.h"
 #include "stringexpressionwriter.h"
@@ -32,15 +33,17 @@
 
 using namespace Analitza;
 
+const double HtmlExpressionWriter::MIN_PRINTABLE_VALUE = 0.0000000000001; // since the precision we choose for 'G' is 12
+
 //we use the one in string*
 QMap<Operator::OperatorType, QString> initOperators();
 
 template <class T>
-QStringList HtmlExpressionWriter::allValues(T it, const T& itEnd, ExpressionWriter* writer)
+QStringList HtmlExpressionWriter::allValues(T it, const T& itEnd, AbstractExpressionVisitor* writer)
 {
 	QStringList elements;
 	for(; it!=itEnd; ++it)
-		elements += (*it)->visit(writer);
+		elements += (*it)->accept(writer).toString();
 	
 	return elements;
 }
@@ -48,58 +51,88 @@ QStringList HtmlExpressionWriter::allValues(T it, const T& itEnd, ExpressionWrit
 
 static const QMap<Operator::OperatorType, QString> s_operators=initOperators();
 
-QString oper(const QString& op) { return i18nc("html representation of an operator", "<span class='op'>%1</span>", op); }
-QString oper(const QChar& op) { return i18nc("html representation of an operator", "<span class='op'>%1</span>", op); }
-QString keyword(const QString& op) { return i18nc("html representation of an operator", "<span class='keyword'>%1</span>", op); }
+QString oper(const QString& op) { return QCoreApplication::translate("html representation of an operator", "<span class='op'>%1</span>").arg(op); }
+QString oper(const QChar& op) { return QCoreApplication::translate("html representation of an operator", "<span class='op'>%1</span>").arg(op); }
+QString keyword(const QString& op) { return QCoreApplication::translate("html representation of an operator", "<span class='keyword'>%1</span>").arg(op); }
 
 HtmlExpressionWriter::HtmlExpressionWriter(const Object* o)
 {
 	if(o)
-		m_result=o->visit(this);
+		m_result=o->accept(this);
 }
 
-QString HtmlExpressionWriter::accept(const Vector* vec)
+QVariant HtmlExpressionWriter::visit(const Vector* vec)
 {
-	return keyword("vector ")+oper("{ ")+allValues<Vector::const_iterator>(vec->constBegin(), vec->constEnd(), this).join(QString(oper(", ")))+oper(" }");
+	return QString(keyword("vector ")+oper("{ ")+allValues<Vector::const_iterator>(vec->constBegin(), vec->constEnd(), this).join(QString(oper(", ")))+oper(" }"));
 }
 
-QString HtmlExpressionWriter::accept(const Matrix* m)
+QVariant HtmlExpressionWriter::visit(const Matrix* m)
 {
-	return keyword("matrix ")+oper("{ ")+allValues(m->constBegin(), m->constEnd(), this).join(QString(oper(", ")))+oper(" }");
+	return QString(keyword("matrix ")+oper("{ ")+allValues(m->constBegin(), m->constEnd(), this).join(QString(oper(", ")))+oper(" }"));
 }
 
-QString HtmlExpressionWriter::accept(const MatrixRow* mr)
+QVariant HtmlExpressionWriter::visit(const MatrixRow* mr)
 {
-	return keyword("matrixrow ")+oper("{ ")+allValues(mr->constBegin(), mr->constEnd(), this).join(QString(oper(", ")))+oper(" }");
+	return QString(keyword("matrixrow ")+oper("{ ")+allValues(mr->constBegin(), mr->constEnd(), this).join(QString(oper(", ")))+oper(" }"));
 }
 
-QString HtmlExpressionWriter::accept(const List* vec)
+QVariant HtmlExpressionWriter::visit(const List* vec)
 {
 	if(!vec->isEmpty() && vec->at(0)->type()==Object::value && static_cast<Cn*>(vec->at(0))->format()==Cn::Char) {
-		return "<span class='string'>&quot;"+AnalitzaUtils::listToString(vec)+ "&quot;</span>";
+		return QString("<span class='string'>&quot;"+AnalitzaUtils::listToString(vec)+ "&quot;</span>");
 	} else
-		return keyword("list ")+oper("{ ")+allValues<List::const_iterator>(vec->constBegin(), vec->constEnd(), this).join(QString(oper(", ")))+oper(" }");
+		return QString((keyword("list ")+oper("{ ")+allValues<List::const_iterator>(vec->constBegin(), vec->constEnd(), this).join(QString(oper(", ")))+oper(" }")));
 }
 
-QString HtmlExpressionWriter::accept(const Cn* var)
+QVariant HtmlExpressionWriter::visit(const Cn* var)
 {
+	QString innerhtml;
 	if(var->isBoolean())
-		return "<span class='var'>"+QString(var->isTrue() ? "true" : "false")+"</span>";
+		innerhtml = var->isTrue() ? "true" : "false";
+	else if(var->isCharacter())
+		innerhtml = QString(var->character());
+	else if(var->isComplex()) {
+		QString realpart;
+		QString imagpart;
+		bool realiszero = false;
+		if (qAbs(var->complexValue().real()) > MIN_PRINTABLE_VALUE)
+			realpart = QString::number(var->complexValue().real(), 'g', 12);
+		else
+			realiszero = true;
+		
+		if (var->complexValue().imag() != 1 && var->complexValue().imag() != -1) {
+			if (qAbs(var->complexValue().imag()) > MIN_PRINTABLE_VALUE) {
+				if (!realiszero && var->complexValue().imag()>0.)
+					realpart += QLatin1String("+");
+				imagpart = QString::number(var->complexValue().imag(), 'g', 12);
+				imagpart += QLatin1String("*i");
+			}
+		} else  {
+			if (var->complexValue().imag() == 1)
+				imagpart = QLatin1String("i");
+			else if (var->complexValue().imag() == -1)
+				imagpart = QLatin1String("-i");
+		}
+		
+		innerhtml = realpart+imagpart;
+	}
 	else
-		return "<span class='num'>"+QString::number(var->value(), 'g', 12)+"</span>";
+		innerhtml = QString::number(var->value(), 'g', 12);
+	
+	return QString("<span class='num'>"+innerhtml+"</span>");
 }
 
-QString HtmlExpressionWriter::accept(const Analitza::Ci* var)
+QVariant HtmlExpressionWriter::visit(const Analitza::Ci* var)
 {
 	return var->toHtml();
 }
 
-QString HtmlExpressionWriter::accept(const Analitza::Operator* o)
+QVariant HtmlExpressionWriter::visit(const Analitza::Operator* o)
 {
-	return "<span class='func'>"+o->toString()+"</span>";
+	return QString("<span class='func'>"+o->toString()+"</span>");
 }
 
-QString HtmlExpressionWriter::accept ( const Analitza::Apply* a )
+QVariant HtmlExpressionWriter::visit ( const Analitza::Apply* a )
 {
 	Operator op=a->firstOperator();
 	QStringList ret;
@@ -109,13 +142,13 @@ QString HtmlExpressionWriter::accept ( const Analitza::Apply* a )
 	if(a->ulimit() || a->dlimit()) {
 		bounds += oper('=');
 		if(a->dlimit())
-			bounds += a->dlimit()->visit(this);
+			bounds += a->dlimit()->accept(this).toString();
 		bounds += oper("..");
 		if(a->ulimit())
-			bounds += a->ulimit()->visit(this);
+			bounds += a->ulimit()->accept(this).toString();
 	}
 	else if(a->domain())
-		bounds += oper('@')+a->domain()->visit(this);
+		bounds += oper('@')+a->domain()->accept(this).toString();
 	
 	int i = 0;
 	foreach(Object* o, a->m_params) {
@@ -125,11 +158,11 @@ QString HtmlExpressionWriter::accept ( const Analitza::Apply* a )
 				Q_ASSERT(false);
 				break;
 			case Object::variable:
-				ret << static_cast<const Ci*>(o)->visit(this);
+				ret << static_cast<const Ci*>(o)->accept(this).toString();
 				break;
 			case Object::apply: {
 				Apply *c = (Apply*) o;
-				QString s = c->visit(this);
+				QString s = c->accept(this).toString();
 				if(s_operators.contains(op.operatorType())) {
 					Operator child_op = c->firstOperator();
 					
@@ -140,7 +173,7 @@ QString HtmlExpressionWriter::accept ( const Analitza::Apply* a )
 				ret << s;
 			}	break;
 			default:
-				ret << o->visit(this);
+				ret << o->accept(this).toString();
 				break;
 		}
 		++i;
@@ -169,7 +202,7 @@ QString HtmlExpressionWriter::accept ( const Analitza::Apply* a )
 		QString bounding;
 		QStringList bvars;
 		foreach(const Ci* bvar, a->bvarCi())
-			bvars += bvar->visit(this);
+			bvars += bvar->accept(this).toString();
 		
 		if(!bounds.isEmpty() || !bvars.isEmpty()) {
 			if(bvars.count()!=1) bounding +=oper('(');
@@ -179,13 +212,13 @@ QString HtmlExpressionWriter::accept ( const Analitza::Apply* a )
 			bounding = ':'+bounding +bounds;
 		}
 		
-		toret += op.visit(this)+oper('(')+ret.join(oper(", "))+bounding+oper(')');
+		toret += op.accept(this).toString()+oper('(')+ret.join(oper(", "))+bounding+oper(')');
 	}
 	
 	return toret;
 }
 
-QString HtmlExpressionWriter::accept(const Container* var)
+QVariant HtmlExpressionWriter::visit(const Container* var)
 {
 	QStringList ret = allValues(var->constBegin(), var->constEnd(), this);
 	
@@ -199,7 +232,7 @@ QString HtmlExpressionWriter::accept(const Container* var)
 			QString last=ret.takeLast();
 			QStringList bvars;
 			foreach(const Ci* bvar, var->bvarCi())
-				bvars += bvar->visit(this);
+				bvars += bvar->accept(this).toString();
 			
 			if(bvars.count()!=1) toret +=oper('(');
 			toret += bvars.join(", ");
@@ -230,7 +263,12 @@ QString HtmlExpressionWriter::accept(const Container* var)
 	return toret;
 }
 
-QString HtmlExpressionWriter::accept(const CustomObject*)
+QVariant HtmlExpressionWriter::visit(const CustomObject*)
 {
-	return "Custom Object";
+	return QString("Custom Object");
+}
+
+QVariant HtmlExpressionWriter::visit(const None* )
+{
+	return QString();
 }
