@@ -25,7 +25,6 @@
 #include <QBitArray>
 #include <QLineF>
 
-#include <analitza/localize.h>
 #include <analitza/value.h>
 #include <analitza/expressiontype.h>
 #include <analitza/variable.h>
@@ -44,7 +43,7 @@ class FunctionImplicit : public AbstractPlaneCurve, public MarchingSquares
 {
 public:
     CONSTRUCTORS(FunctionImplicit)
-    TYPE_NAME(I18N_NOOP("Implicit Curve"))
+    TYPE_NAME(QT_TR_NOOP("Implicit Curve"))
     EXPRESSION_TYPE(Analitza::ExpressionType(Analitza::ExpressionType::Lambda)
         .addParameter(Analitza::ExpressionType(Analitza::ExpressionType::Value))
         .addParameter(Analitza::ExpressionType(Analitza::ExpressionType::Value))
@@ -71,28 +70,22 @@ void FunctionImplicit::update(const QRectF& vp)
     points.clear();
     jumps.clear();
 
-    //TODO
-//     QPair<double, double> ix = interval("x");
-//     QPair<double, double> iy = interval("y");    
-//     double minx = ix.first;
-//     double maxx = ix.second;
-//     double miny = iy.first;
-//     double maxy = iy.second;
-//     
+    double minx = vp.left();
+    double maxx = vp.right();
+    double miny = vp.top();
+    double maxy = vp.bottom();
     
-    double minx = 0;
-    double maxx = 0;
-    double miny = 0;
-    double maxy = 0;
-    
-    if (!hasIntervals())
+    if (hasIntervals())
     {
-        minx = vp.left();
-        maxx = vp.right();
-        miny = vp.top();
-        maxy = vp.bottom();
-    }
+        QPair<double, double> intervalx = interval("x");
+        QPair<double, double> intervaly = interval("y");
 
+        minx = intervalx.first;
+        maxx = intervalx.second;
+        miny = intervaly.first;
+        maxy = intervaly.second;
+    }
+    
     setWorld(minx, maxx, miny, maxy);
     buildGeometry();
 
@@ -102,14 +95,14 @@ void FunctionImplicit::update(const QRectF& vp)
         jumps.append(points.size());
     }
 
-    if (points.size() <= 2) // y aunque/PESE a que el viewport se corta con el dominio
-    {
-//         appendError(i18nc("This function can't be represented as a curve. To draw implicit curve, the function has to satisfy the implicit function theorem.", "Implicit function undefined in the plane"));
-    }
+//     if (points.size() <= 2) // y aunque/PESE a que el viewport se corta con el dominio
+//     {
+//         appendError(QCoreApplication::translate("This function can't be represented as a curve. To draw implicit curve, the function has to satisfy the implicit function theorem.", "Implicit function undefined in the plane"));
+//     }
 }
 
 //Own
-QPair<QPointF, QString> FunctionImplicit::image(const QPointF &point)
+QPair<QPointF, QString> FunctionImplicit::image(const QPointF &/*point*/)
 {
 
     return qMakePair(QPointF(), QString());
@@ -223,7 +216,7 @@ QPair<QPointF, QString> FunctionImplicit::image(const QPointF &point)
 
 }
 
-QLineF FunctionImplicit::tangent(const QPointF &mousepos)
+QLineF FunctionImplicit::tangent(const QPointF &/*mousepos*/)
 {
     return QLineF();
 }
